@@ -23,18 +23,46 @@
 #  USA
 
 import apt_pkg
-from AptAuth import AptAuth
-from gettext import gettext as _
+from hashlib import md5
+import re
 import os
+import glob
+import sys
+import shutil
+import subprocess
+from tempfile import NamedTemporaryFile
 import string
 from xml.sax.saxutils import escape
-import aptsources
-from aptsources.sourceslist import SourcesList, SourceEntry
-import softwareproperties
-import md5
-import re
+from ConfigParser import ConfigParser
+from gettext import gettext as _
 
-class SoftwareProperties:
+from AptAuth import AptAuth
+import softwareproperties
+import aptsources
+import aptsources.distro
+from aptsources.sourceslist import SourcesList, SourceEntry
+
+
+
+class SoftwareProperties(object):
+
+  # known (whitelisted) channels
+  CHANNEL_PATH="/usr/share/app-install/channels/"
+
+  # release upgrades policy
+  RELEASE_UPGRADES_CONF = "/etc/update-manager/release-upgrades"
+  #RELEASE_UPGRADES_CONF = "/tmp/release-upgrades"
+  (
+    RELEASE_UPGRADES_NEVER,
+    RELEASE_UPGRADES_NORMAL,
+    RELEASE_UPGRADES_LTS
+  ) = range(3)
+  release_upgrades_policy_map = {
+    RELEASE_UPGRADES_NEVER  : 'never',
+    RELEASE_UPGRADES_NORMAL : 'normal',
+    RELEASE_UPGRADES_LTS    : 'lts',
+  }
+  
   def __init__(self, datadir=None, options=None):
     """ Provides the core functionality to configure the used software 
         repositories, the corresponding authentication keys and 
@@ -132,6 +160,49 @@ class SoftwareProperties:
     """ Returns the interval of the apt periodic cron job """
     return apt_pkg.Config.FindI(softwareproperties.CONF_MAP["autoupdate"])
 
+  def get_release_upgrades_policy(self):
+    """
+    return the release upgrade policy:
+     RELASE_UPGRADE_NEVER
+     RELASE_UPGRADE_NORMAL
+     RELASE_UPGRADE_LTS
+    """
+    # default (if no option is set) is NORMAL
+    if not os.path.exists(self.RELEASE_UPGRADES_CONF):
+      return self.RELEASE_UPGRADES_NORMAL
+    parser = ConfigParser()
+    parser.read(self.RELEASE_UPGRADES_CONF)
+    if parser.has_option("DEFAULT","Prompt"):
+      type = parser.get("DEFAULT","Prompt").lower()
+      for k, v in self.release_upgrades_policy_map.iteritems():
+        if v == type:
+          return k
+    return self.RELEASE_UPGRADES_NORMAL
+
+  def set_release_upgrades_policy(self, i):
+    """
+    set the release upgrade policy:
+     RELASE_UPGRADE_NEVER
+     RELASE_UPGRADE_NORMAL
+     RELASE_UPGRADE_LTS
+     """
+    # we are note using ConfigParser.write() as it removes comments
+    if not os.path.exists(self.RELEASE_UPGRADES_CONF):
+      f = open(self.RELEASE_UPGRADES_CONF,"w")
+      f.write("[DEFAULT]\nprompt=%s\n"% self.release_upgrades_policy_map[i])
+      return True
+    f = open(self.RELEASE_UPGRADES_CONF,"r")
+    out = NamedTemporaryFile()
+    for line in map(string.strip, f.readlines()):
+      if line.lower().startswith("prompt"):
+        out.write("prompt=%s\n" % self.release_upgrades_policy_map[i])
+      else:
+        out.write(line+"\n")
+    out.flush()
+    shutil.copymode(self.RELEASE_UPGRADES_CONF, out.name)
+    shutil.copy(out.name, self.RELEASE_UPGRADES_CONF)
+    return True
+
   def get_popcon_participation(self):
     """ Will return True if the user wants to participate in the popularity 
         contest. Otherwise it will return False. Special case: if no 
@@ -165,7 +236,7 @@ class SoftwareProperties:
                     open(self.popconfile, "r").readlines())
     else:
         # create a new popcon config file
-        m = md5.new()
+        m = md5()
         m.update(open("/dev/urandom", "r").read(1024))
         id = m.hexdigest()
         lines = []
@@ -255,7 +326,13 @@ class SoftwareProperties:
 
   def enable_child_source(self, template):
     """Enable a child repo of the distribution main repository"""
-    self.distro.add_source(uri=template.base_uri, dist=template.name)
+    # Use the currently selected mirror only if the child source
+    # did not override the server
+    if template.base_uri == None:
+        child_uri = self.distro.default_server
+    else:
+        child_uri = template.base_uri
+    self.distro.add_source(uri=child_uri, dist=template.name)
     self.set_modified_sourceslist()
 
   def disable_source_code_sources(self):
@@ -485,8 +562,39 @@ class SoftwareProperties:
     self.sourceslist.backup(".save")
     self.sourceslist.save()
 
+  def _is_line_in_whitelisted_channel(self, srcline):
+    """
+    helper that checks if a given line is in the source list
+    return the channel name or None if not found
+    """
+    srcentry = SourceEntry(srcline)    
+    if os.path.exists(self.CHANNEL_PATH):
+      for f in glob.glob("%s/*.list" % self.CHANNEL_PATH):
+        for line in open(f):
+          if line.strip().startswith("#"):
+            continue
+          if srcentry == SourceEntry(line):
+            return os.path.splitext(os.path.basename(f))[0]
+    return None
+
+  def check_and_add_key_for_whitelisted_channels(self, srcline):
+    """
+    helper that adds the gpg key of the channel to the apt
+    keyring *if* the channel is in the whitelist
+    /usr/share/app-install/channels
+    """
+    channel = self._is_line_in_whitelisted_channel(srcline)
+    if channel:
+      keyp = "%s/%s.key" % (self.CHANNEL_PATH, channel)
+      if os.path.exists(keyp):
+        subprocess.call(["apt-key","add",keyp])
+
   def add_source_from_line(self, line):
-    """Add a source with the given apt line"""
+    """
+    Add a source with the given apt line and auto-add
+    signing key if we have it in the whitelist
+    """
+    self.check_and_add_key_for_whitelisted_channels(line)
     self.sourceslist.list.append(SourceEntry(line))
     self.set_modified_sourceslist()
 
@@ -512,3 +620,10 @@ class SoftwareProperties:
         return True
     except:
         return False
+
+
+
+if __name__ == "__main__":
+  sp = SoftwareProperties()
+  print sp.get_release_upgrades_policy()
+  sp.set_release_upgrades_policy(0)
